/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.web.user;

import cn.dev33.satoken.stp.StpUtil;
import com.yiring.auth.domain.role.Role;
import com.yiring.auth.domain.role.RoleRepository;
import com.yiring.auth.domain.user.User;
import com.yiring.auth.domain.user.UserRepository;
import com.yiring.auth.param.IdsParam;
import com.yiring.auth.util.Permissions;
import com.yiring.auth.vo.user.UserInfoVo;
import com.yiring.auth.vo.user.UserVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 系统用户控制器
 *
 * @author Jim
 * @version 0.1
 * 2022/1/24 14:13
 */

@Slf4j
@Validated
@Api(tags = "User")
@RestController
@RequestMapping("/user/")
public class UserController {

    @Resource
    UserRepository userRepository;

    @Resource
    RoleRepository roleRepository;

    @ApiOperation(value = "获取登录用户信息")
    @GetMapping("info")
    public Result<UserInfoVo> info() {
        Long id = StpUtil.getLoginIdAsLong();
        Optional<User> optional = userRepository.findById(id);
        if (optional.isEmpty()) {
            StpUtil.logout(id);
            return Result.no(Status.UNAUTHORIZED);
        }

        User user = optional.get();
        UserInfoVo userInfoVo = UserInfoVo
            .builder()
            .userId(user.getId())
            .realName(user.getRealName())
            .avatar(user.getAvatar())
            .title(user.getTitle())
            .roles(Permissions.toRoleVos(user.getRoles()))
            .permissions(Permissions.toTree(Permissions.toPermissions(user.getRoles()), 0L))
            // 默认跳转到用户看板
            .homePath("/dashboard/workbench")
            .build();
        return Result.ok(userInfoVo);
    }

    @ApiOperation(value = "分配角色")
    @PostMapping("/manage/assign")
    public Result<String> assign(@Valid IdParam idParam, @Valid IdsParam idsParam) {
        Optional<User> optional = userRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        // 查询权限集合
        Set<Long> ids = idsParam.toIds();
        Set<Role> roles = roleRepository
            .findAll()
            .stream()
            .filter(role -> ids.contains(role.getId()))
            .collect(Collectors.toSet());

        User entity = optional.get();
        entity.setRoles(roles);
        userRepository.saveAndFlush(entity);
        return Result.ok();
    }

    @ApiOperation(value = "分页查询")
    @GetMapping("/manage/page")
    public Result<PageVo<UserVo>> page(@Valid PageParam param) {
        Page<User> page = userRepository.findAll(PageParam.toPageable(param));
        List<UserVo> data = page
            .get()
            .map(role -> {
                UserVo vo = new UserVo();
                BeanUtils.copyProperties(role, vo);
                return vo;
            })
            .collect(Collectors.toList());
        PageVo<UserVo> vo = PageVo.build(data, page.getTotalElements());
        return Result.ok(vo);
    }
}
